/* File name : main.pl */ 

/************************************************************************
Natural Language Processing - Assignment
Written by : Shoo Kok Keong (MC9494)
File included : main.pl  (main file)
		atn.pl   (checking grammar using atn method)
		dcg.pl   (checking grammar using dcg method)
		bond.txt (file to be checked)
		vocab.pl (Contain all the vocabularies used)
Written in : 24/03/98
Last Modified : 30/04/98

Type 'go' to start.

Some common functions used in all file are stored in this file (main.pl).
************************************************************************/

go:-consult('vocab.pl'),go1.

go1:-put(12),draw_menu,draw_main_option,
	write(' Enter you choice '),
	get_choice(Choice),go1(Choice).

go1(49):-consult('dcg.pl'),dcg,go1.
go1(50):-consult('atn.pl'),atn,go1.  
go1(51):-draw_end.

/* Validate the choice */
get_choice(X):-
	get0(X),check(X). 
get_choice(X):-get_choice(X). 

check(49). /* 1 */    
check(50). /* 2 */
check(51). /* 3 */

draw_menu:-
	write('============================================'),nl,
	write('University of Portsmouth'),nl,
	write('Natural Language Processing - Assignment'),nl,
	write('Grammar Checking System version 1.0'),nl,
	write('Written by Shoo Kok Keong'),nl,
	write('============================================'),nl,nl.

draw_main_option:-
	write('  1 - Grammar checking using DCG.'),nl,
	write('  2 - Grammar checking using ATN (incomplete version!).'),nl,
	write('  3 - Exit'),
	nl,nl,nl.

draw_end:-nl,nl,
	write('Thank you for using this system.'),nl,
	write('Goodbye.'),nl.

/* Common rules or functions used in both 'atn.pl' and 'dcg.pl' */

/* Write out states and parts of sentence */ 

write_parse:-nl,record(X,Y),tab(12),write(Y),tab(12),
        write(X),nl,fail.
write_parse:-nl,retractall(record(_,_)).

/* Reading in a normal sentence as a list - modified slightly
   to detect end of file */

getsent(_,[W|Ws]):-get0(C),not C=(-1),
        readword(C,W,C1),
        restsent(W,C1,Ws).
getsent(File,[eof]).

restsent(W,_,[]):-lastword(W),!.
restsent(W,C,[W1|Ws]):-readword(C,W1,C1),
        restsent(W1,C1,Ws).

readword(C,W,C1):-single_character(C),
        !,name(W,[C]),
        get0(C1),not C1=(-1).
readword(C,W,C2):-in_word(C,NewC),!,
        get0(C1),not C1=(-1),
        restword(C1,Cs,C2),Z=[NewC|Cs],name(W,Z).
readword(C,W,C2):-
        get0(C1),not C1=(-1),
        readword(C1,W,C2).

restword(C,[NewC|Cs],C2):-in_word(C,NewC),!,
        get0(C1),not C1=(-1),
        restword(C1,Cs,C2).
restword(C,[],C).

single_character(44).    single_character(59).    
single_character(58).    single_character(63).    
single_character(33).    single_character(46).

in_word(C,C):-C>96,C<123.     in_word(C,C):-C>47,C<58.
in_word(39,39).               in_word(45,45).
in_word(C,L):-C>64,C<91,L is C+32.

lastword('.').      lastword('!').      lastword('?').

/* Writing out a list of words as a normal sentence */

writesent([L|Lt]):-name(L,Q),Q=[X|Xt],Z is X-32,R=[Z|Xt],name(Y,R),
        write(Y),writesent2(Lt).

writesent2([]):-write('.'),nl,!.
writesent2([L|Lt]):-L=',',write(L),writesent2(Lt),!.
writesent2([L|Lt]):-write(' '),write(L),writesent2(Lt).
                             
/* Remove_fullstop */

remove_fullstop(L1,L2):-reverse(L1,[_|L]),reverse(L,L2).

/* Tailors list to remove adverb and comma */

tailor([],X,Y):-reverse(X,Y),!.
tailor([X|Sent],RunSent,NewSent):-adverb(X),
     tailor(Sent,RunSent,NewSent),!.
tailor([X|Sent],RunSent,NewSent):-tailor(Sent,[X|RunSent],NewSent).

/* Display Mark as integer number */

display_mark(Mark):-name(Mark,List),display(List,_).
display([L|Lt],_):-not L = 46, name(Num,[L]),write(Num), display(Lt,_).
display(_,_).

/* Display ending message, including total incorrect senteces, 
	total sentences checked and total marks */

ending(Incorrect,TotalSent):-write('Total Incorrect Sentences = '),
	write(Incorrect),nl,nl,
	write('Total Sentences Checked = '),
	write(TotalSent),nl,nl,
	M is TotalSent - Incorrect,
	Mark is M / TotalSent * 100,
	write('Total Marks = '),
	display_mark(Mark),write('%'),nl,nl,
	write('Completed.'),nl,
	write('Please scroll up to view result or'),nl,
	write('Press any key to continue.....'),
	getb(ANY).


/* End */
