
/* File name : atn.pl */

/* PROGRAM TO READ IN A TEXT FILE AND ANALYSE SENTENCES WHILE
   READING IN, USING GETSENT */

/* WINPRO VERSION USING -1 TO END FILE */

/* Reads in a text file using 'getsent', converting each sentence in
   the text to a list of words, saving each list as a 'sentlist' fact.
   Writes sentences out from the lists using 'writesent'. */

atn:-   put(12),draw_menu,
	Incorrect is 0,TotalSent is 0, 
        File='bond.txt',
        see(File),
	analyse_atn(File,Sent,Incorrect,TotalSent),
	seen.

analyse_atn(File,Sent,Incorrect,TotalSent):-
	getsent(File,Sent),not Sent=[eof],
        assertz(sentlist(Sent)),
	tailor(Sent,[],NewSent),
        remove_fullstop(NewSent,Sent2),
        go2_atn(Sent2,Incorrect,TotalSent),!.
analyse_atn(_,[eof],Incorrect,TotalSent):-ending(Incorrect,TotalSent).

go2_atn(Sent,Incorrect,TotalSent):-
	test_atn(Sent,Incorrect,TotalSent).
go2_atn(Sent,Incorrect,TotalSent):-retractall(record('State',_)),
	write_parse,
	retractall(record(_,_)),
	writesent(Sent),N1 is Incorrect + 1,T1 is TotalSent + 1,
        write('Incorrect grammar.'),nl,nl,analyse_atn(_,_,N1,T1).

test_atn(Sent,Incorrect,TotalSent):-new_state(start),
	transition(Sent,Rest),
	Rest = [],retractall(record('State',_)),
        write_parse,
        nl,nl,
	writesent(Sent),T1 is TotalSent + 1,
        write('grammar.'),nl,nl,analyse_atn(_,_,Incorrect,T1).

transition([],Rest).
transition(Sent,Rest):-state(start),sentence(Sent,Next),new_state(s1),transition(Next,Rest),!.
transition(Sent,Rest):-state(s1),is_coord_conj(Sent,Next),state(found),new_state(start),
	sentence(Next,Next1),new_state(s2),transition(Next1,Rest),!.
transition(Sent,Rest):-state(s2),new_state(start),verb_prep_phrase(Sent,Next),
	new_state(s3),transition(Next,Rest),!.
transition(Sent,Rest):-state(s4),is_coord_conj(Sent,Next),new_state(start),
	verb_prep_phrase(Next,Next1),new_state(end),transition(Next1,Rest),!.
transition([L|Lt],Rest):-state(end),register(unknown,L),new_state(incorrect),transition(Lt,Rest),!.
transition(Sent,Sent).

is_coord_conj([L|Lt],Rest):-coord_conj(L), register(coord_conj,L),new_state(found),
	is_coord_conj(Lt,Rest),!.
is_coord_conj(Sent,Sent).
	
sentence([],Rest).
sentence(Sent,Rest):-state(start),noun_phrase(Sent,Next),new_state(start),
	verb_prep_phrase(Next,Next1),new_state(end),sentence(Next1,Rest),!.
sentence(Sent,Sent).

noun_phrase([],Rest).
noun_phrase(Sent,Rest):-state(start),noun_adj_group(Sent,Next),
	new_state(s2),noun_phrase(Next,Rest),!.
noun_phrase(Sent,Rest):-state(s2),adverbial_phrase(Sent,Next),new_state(s3),
	noun_phrase(Next,Rest),!.
noun_phrase([L|Lt],Rest):-propernoun(L),register(propernoun,L),
        new_state(start), noun_phrase(Lt,Rest),!.
noun_phrase([L|Lt],Rest):-pronoun(L),register(pronoun,L),
        new_state(start),noun_phrase(Lt,Rest),!.
noun_phrase([L|Lt],Rest):-adjective(L),register(adjective,L),
        new_state(start),noun_phrase(Lt,Rest),!.
noun_phrase([L|Lt],Rest):-determiner(L),register(determiner,L),
	new_state(start),noun_phrase(Lt,Rest),!.
noun_phrase([L|Lt],Rest):-genitive(L),register(genitive,L),
        new_state(start),noun_phrase(Lt,Rest),!.
noun_phrase(Sent,Rest):-state(s3),unknown(Sent),
        new_state(end),noun_phrase(Sent,Rest),!.
noun_phrase(Sent,Sent).

unknown([L|Lt]):-retractall(record(unknown,L)),register(unknown,L).

noun_adj_group([],Rest).
noun_adj_group([L|Lt],Rest):-adjective(L),register(adjective,L),
        noun_adj_group(Lt,Rest),!.
noun_adj_group([L|Lt],Rest):-noun(L),register(noun,L),
        noun_adj_group(Lt,Rest),!.
noun_adj_group(Sent,Sent).

verb_prep_phrase([],Rest).
verb_prep_phrase(Sent,Rest):-state(start),verb_phrase(Sent,Rest),
	new_state(v1),verb_prep_phrase(Rest,Rest),!.
verb_prep_phrase(Sent,Rest):-state(v1),prepositional_phrase(Sent,Rest),
	new_state(v2),verb_prep_phrase(Rest,Rest),!.
verb_prep_phrase(Sent,Sent).

verb_group([L|Lt],Rest):- verb(L), register(verb,L),
	verb_group(Lt,Rest),!.
verb_group([L|Lt],Rest):- participle(L), register(participle,L),
	verb_group(Lt,Rest),!.
verb_group([L|Lt],Rest):- pronoun(L), register(pronoun,L),
	verb_group(Lt,Rest),!.
verb_group(Sent,Sent).

verb_phrase([],Rest).
verb_phrase(Sent,Rest):-state(start),verb_group(Sent, Next), 
	new_state(s1),verb_phrase(Next,Rest),!.
verb_phrase(Sent,Rest):-state(s1),new_state(start),noun_phrase(Sent, Next), 
	new_state(s2),verb_phrase(Next,Rest),!.
verb_phrase(Sent,Rest):-state(s2),prepositional_phrase(Sent, Next), 
	new_state(s3),verb_phrase(Next,Rest),!.
/*verb_phrase([L|Lt],Rest):-infinitive(L), register(infinitive,L),
	verb_phrase(Lt,Rest),!.
*/
verb_phrase(Sent,Sent).

prepositional_phrase([],Rest).
prepositional_phrase(Sent,Rest):-state(s2),adverbial_phrase(Sent,Next),new_state(s3),
	prepositional_phrase(Next,Rest),!.
prepositional_phrase([L|Lt],Rest):-preposition(L), register(preposition,L),
	prepositional_phrase(Lt,Rest),!.
prepositional_phrase(Sent,Rest):-state(s3),new_state(start),noun_phrase(Sent,Rest),
	new_state(new),prepositional_phrase(Rest,Rest),!.
prepositional_phrase(Sent,Sent).

/*adverbial_phrase(Sent,Rest):-isadverbial(Sent,Next),fail.
adverbial_phrase(Sent,Sent).
*/

adverbial_phrase(Sent,Rest):-getnum(1,Sent,L),adverbial(L),
	register(adverbial,L),switch(Sent,Next,2),
	adverbial_phrase(Next,Rest),!.
adverbial_phrase(Sent,Rest):-getnum(1,Sent,L),getnum(2,Sent,Term),
	adverbial(L,Term),register(adverbial,L),
	switch(Sent,Next,3),adverbial_phrase(Next,Rest),!.
adverbial_phrase(Sent,Rest):-getnum(1,Sent,L),getnum(2,Sent,Term),
	getnum(3,Sent,Term2),adverbial(L,Term,Term2),
	switch(Sent,Next,4),adverbial_phrase(Next,Rest),!.
adverbial_phrase(Sent,Rest):-getnum(1,Sent,L),getnum(2,Sent,Term),
	getnum(3,Sent,Term2),getnum(4,Sent,Term3),
	adverbial(L,Term,Term2,Term3),switch(Sent,Next,5),
	adverbial_phrase(Next,Rest),!.
adverbial_phrase(Sent,Sent).

adverbial(at,once).
adverbial(out,of,bed).
adverbial(from,side,to,side).
adverbial(in,agony).
adverbial(death).
adverbial(out,of,sight).

/* getnum(N,List,Term) gets term number N from list List. */ 
getnum(N,L,T):-get_term(N,L,T,1).
get_term(N,[L|_],L,N).
get_term(N,[_|Lt],X,M):-Q is M+1,get_term(N,Lt,X,Q).

/* switch(List,NewList,Number) gets the element number (Number) in the list 
   (List) to become the first element in new list NewList */

switch([L|Lt],New,Num):- N is Num - 1, N > 0,switch(Lt,New,N).
switch(New,New,1).

/* Record states and parts of sentence */
new_state(Q):-retractall(state(_)),assertz(state(Q)),
        assertz(record('State',Q)).

register(Part,Word):-assertz(record(Part,Word)).

/* END */

